.extern __bss_start__, __bss_end__, _etext, _sdata
.extern __bss_start, __end__, _edata, _end, _stack, _etext

RAM_BASE = 0x02000000
RAM_END  = 0x02040000

Cpu_Mode_USR = 0x10
Cpu_Mode_IRQ = 0x12
Cpu_Mode_SVC = 0x13

Cpu_Stack_USR = 0x03007F00
Cpu_Stack_IRQ = 0x03007FA0
Cpu_Stack_SVC = 0x03007FE0

INTERRUPT_VECTOR = 0x03007FFC

.globl _start

.text

_start:	b	reset
.include "header.s"

reset:
	mov	r0, #(Cpu_Mode_IRQ | 0x80 | 0x40)
	#msr	CPSR_c, r0
	ldr	r13, =Cpu_Stack_IRQ

	mov	r0, #(Cpu_Mode_SVC | 0x80 | 0x40)
	#msr	CPSR_c, r0
	ldr	r13, =Cpu_Stack_SVC

	@@@ Code to copy DATA into RAM. 
	ldr	r0, =_sdata		@ i.e., _sdata = 0x08001ef0
	ldr	r1, =RAM_BASE		@ r1 = 0x03000000
	ldr	r3, =_edata	@ r3 = 0x02000100 (or however big data/bss is)

.copy:	cmp	r1, r3
	ldrcc	r2, [r0], #4
	strcc	r2, [r1], #4
	bcc	.copy
.zero:	mov	r3, #RAM_END	
	mov	r2, #0
.znxt:	cmp	r1, r3
	strcc	r2, [r1], #4
	bcc	.znxt

	mov	r0, #(Cpu_Mode_SVC | 0x40)
	#msr	CPSR_c, r0

	mov     r0, #(Cpu_Mode_USR | 0x80 | 0x40)
	#msr	CPSR_c, r0

	@ default irq handler 
	ldr	r0, =INTERRUPT_VECTOR
	adr	r1, default_irq
	str	r1, [r0]
	
	mov	r0, #(Cpu_Mode_USR | 0x40)	@ enable interrupts
	#msr	CPSR_c, r0
	ldr	sp, =Cpu_Stack_USR

	bl	main

@ on return
done:	swi	#3	@ gba sys comand - halt
	bal	done	@ on return

default_irq:
	.long 0xE12FFF1E	@ bx lr
